namespace eval atlantis {}

# Atlantis - Gnre des statistiques IRC pour des canaux
# ==========
# Auteur: Merwin
# Version: 0.3
# ==========
# Pour tout support, visitez le site http://www.eggdrop-france.net.
# Documentation sur le site et forum  votre disposition !
# Merci de ne pas redistribuer ce script sans l'accord de son auteur.
# ==========
# Liste des commandes:
#   o !stats
#     Affiche les statistiques de l'utilisateur qui tape la commande.
#   o !top10 [lignes|mots|lettres|sourires|grimaces|questions|kick|kicked|topic]
#     Affiche le top10 des utilisateurs class par catgorie.
#   o !statsinfos
#     Affiche des informations sur le script.
# ==========

# ============================
# = CONFIGURATION DU SCRIPT =
# ============================

# Nombre de secondes que l'utilisateur doit attendre pour utiliser deux commandes
# publique  la suite.
set atlantis(antiflood) 5

# Liste des canaux ou le script doit faire les statistiques, spars par des
# espaces.
set atlantis(channels) "#akseo"

# Nombre de minutes qu'un utilisateur doit rester au minimum sur le canal
# pour que des statistiques sois cres sur lui.
set atlantis(minimum) 15

# Activer la protection anti-flood des stats ? Permet de ne pas tenir compte
# d'un flood dans les statistiques. (yes/no)
set atlantis(anti_statsflood) yes

# Activer les commandes publiques !stats, !top10 et !top20 ? (yes/no)
set atlantis(allow_pubcommands) yes

# Nom du fichier o sont stocks les informations des statistiques, ne pas
# spcifier de chemin relatif, juste le nom.
set atlantis(db_fname) "atlantis.db"

# Comment communiquer avec l'utilisateur ? Mettez 'NOTICE' pour envoyer les
# messages en notice et 'PRIVMSG' pour envoyer en priv et 'CHAN' pour envoyer
# sur le canal.
set atlantis(say) "CHAN"

# Rpertoire o seront places les pages HTML cres pour les canaux
# Le nom des pages sera canal.html, par exemple eggdrop.html pour #eggdrop.
set atlantis(html_dir) "/home/Merwin/public_html/"

# Nom du fichier de templates pour l'affichage HTML (c'est le fichier de
# modle, il DOIT exister dans le dossier templates/  la racine du script)
set atlantis(template) "eggdrop-france.template"

# Les utilisateurs qui n'ont pas t vu depuis le nombre de jours de dfinis
# ici seront automatiquement supprims.
set atlantis(autodelete) 7


# =============================================================================
# = NE PAS MODIFIER APRES CETTE LIGNE SI VOUS NE SAVEZ PAS CE QUE VOUS FAITES =
# =============================================================================

set i 0
set atlantis(channels) [string tolower $atlantis(channels)]
set atlantis(version) 0.4
set atlantis(users_stats) {PSEUDO LIGNES MOTS LETTRES SOURIRES GRIMACES KICK QUESTIONS ACTIONS TOPIC KICKED URL}

foreach s $atlantis(users_stats) {
  set stats_corresp($s) $i
  incr i
}

proc atlantis::scriptInit {args} {
  global atlantis
  
  # Initialisation du script, fix de certaines variables et cration de la
  # 'base' si elle n'existe pas.
  
  set atlantis(working_dir) [file dirname [info script]]
  set atlantis(data_dir) [file join $atlantis(working_dir) "data"]
  set atlantis(db_file) [file join $atlantis(data_dir) $atlantis(db_fname)]
  set atlantis(template_dir) [file join $atlantis(working_dir) "templates"]
  set atlantis(template_file) [file join $atlantis(template_dir) $atlantis(template)]
  
  if ![file isdir $atlantis(data_dir)] {
    file mkdir $atlantis(data_dir)
  }
  
  if ![file exists $atlantis(template_file)] {
    putlog "Atlantis: Impossible d'initialiser le script: le fichier template est manquant !"
    return -code return
  }
  
  loadStats
  repareStats
  
  bind pubm - "*" atlantis::pubmStats
  bind nick - "*" atlantis::nickStats
  bind pub - "!stats" atlantis::pubStats
  bind pub - "!top10" atlantis::pubTop
  bind pub - "!statsinfos" atlantis::pubStatsInfos
  bind time - "* * * * *" atlantis::timerStats
  bind kick - "*" atlantis::kickStats
  bind topc - "*" atlantis::topicStats
}

proc atlantis::loadStats {{fileName ""}} {
  variable users; global atlantis
  
  # Charge la 'base' de donne depuis le disque dans la variable 'users', ne
  # retourne rien.
  
  if {$fileName == ""} { set fileName $atlantis(db_file) }
  
  if ![file exists $fileName] {
    return
  }
  
  set fileID [open $fileName]
  array set users [gets $fileID]
  close $fileID
}

proc atlantis::repareStats {{fileName ""}} {
  variable users; global atlantis
  
  # Rpare / Met  jour la base de donne, utile lorsque l'on change de version
  # et que l'on souhaite conserver ses statistiques d'avant.
  
  set UsersList [array names atlantis::users -regexp {^[^,]+$}]
    
  foreach userName $UsersList {
    
    foreach item $atlantis(users_stats) {
      
      if {![info exists users($userName,$item)]} {
        set users($userName,$item) 0
      }
      
    }
    
  }
}

proc atlantis::saveStats { {fileName ""} } {
  variable users; global atlantis
  
  # Enregistre la 'base' de donne sur le disque dur, si 'fileName' est
  # spcifi c'est cet emplacement, sinon celui de la conf.
  
  if {$fileName == ""} {set fileName $atlantis(db_file)}
  
  set fileID [open $fileName w+]
  puts -nonewline $fileID [array get users]
  close $fileID
}

proc atlantis::getUserName {nick uhost channel} {
  variable users; global atlantis
  
  # Retourne un identifiant unique d'utilisateur qui l'identifie, l'identifiant
  # est de la forme suivante: 3 premires lettre du pseudo, ident et un bout
  # de l'host suivit du canal sans le #.
  
  set uhost [split $uhost "@"]
  set ident [lindex $uhost 0]
  set host [lindex $uhost 1]
  
  append userName [string range $nick 0 2]
  append userName [expr {[string index $ident 0] == "~" ? [string range $ident 1 end] : [set ident]}]
  append userName [join [lrange [split $host "."] end-1 end] ""]
  append userName $channel
  
  return [string tolower $userName]
}

proc atlantis::userExists {userName} {
  variable users
  
  # Retourne 1 si l'utilisateur existe, 0 s'il n'existe pas
  
  return [info exists users($userName)]
}

proc atlantis::userAdd {userName userNick userChannel} {
  variable users; global atlantis
  
  # Cration d'un nouvel utilisateur, on ajoute une entre dans le tableau users
  # Il est de la forme suivante: users(nom,champ) o champ peut tre n'importe
  # quoi (utile pour l'extension).
  
  set userJoined [getchanjoin $userNick $userChannel]
  set inSince [expr {([clock seconds] - $userJoined) / 60}]
  
  if {($inSince <= $atlantis(minimum)) && ($userJoined != 0)} {
    return 0
  }
  
  if [userExists $userName] { return 0 }
  
  foreach stats $atlantis(users_stats) {
    set users($userName,$stats) 0
  }
  
  set users($userName) [unixtime]
  set users($userName,PSEUDO) $userNick
  
  return 1
}

proc atlantis::userDel {userName} {
  variable users
  
  # Supprime une utilisateur et toutes ses statistiques
  
  if [userExists $userName] {
    array unset users $userName*
  }
}

proc atlantis::pubmStats {nick uhost handle channel texte} {
  variable users; global antiflood atlantis
  
  # Modifie les statistiques de l'utilisateur en fonction de la phrase qu'il
  # vient de prononcer.
  
  set userName [getUserName $nick $uhost $channel]
  
  if ![isStatsChan $channel] { return }
  
  if { ![userExists $userName] && ![userAdd $userName $nick $channel] } {
    return
  }
  
  if {[info exists antiflood(text,$userName)] && $atlantis(anti_statsflood)} {
    return
  }

  set happySmileysExpr {(:\))|(:-\))|(:p)|(:D)|(:x)|(:o)|(:\])|(:=\))|(=\))|(;\])}
  set sadSmileysExpr {(:\()|(:-\()|(-_-)|(o_o)}

  # PSEUDO LIGNES MOTS LETTRES SOURIRES GRIMACES KICK QUESTIONS ACTIONS TOPIC

  incr users($userName,LIGNES)
  incr users($userName,MOTS) [llength [split $texte]]
  incr users($userName,LETTRES) [regexp -all -- {[a-zA-Z0-9]} $texte]
  incr users($userName,SOURIRES) [regexp -all -- $happySmileysExpr $texte]
  incr users($userName,GRIMACES) [regexp -all -- $sadSmileysExpr $texte]
  incr users($userName,QUESTIONS) [regexp -- {[\?]} $texte]
  incr users($userName,URL) [regexp -- {(http://)} $texte]
  set users($userName) [unixtime]
  
  set antiflood(text,$userName) 1
  utimer 1 [list "unset" "::antiflood(text,$userName)"]
}

proc atlantis::pubStats {nick uhost handle channel arg} {
  variable users; global antiflood atlantis
  
  # Affiche les statistiques d'un utilisateur sur le canal, incorpore la
  # protection anti-flood pour viter l'abus de la commande.
  
  if {!$atlantis(allow_pubcommands)} { return }
  if ![isStatsChan $channel] { return }
  if [info exists antiflood(cmd,$nick)] { return }
  
  set uName [getUserName $nick $uhost $channel]
  
  set antiflood(cmd,$nick) 1
  utimer $atlantis(antiflood) [list "unset" "::antiflood(cmd,$nick)"]
  
  if ![userExists $uName] {
    sendText "Aucune statistique enregistre pour vous." $nick $channel
    return
  }
  
  sendText "$users($uName,PSEUDO): $users($uName,LIGNES) lignes, $users($uName,MOTS) mots,\
    $users($uName,LETTRES) lettres, $users($uName,SOURIRES) sourires et\
    $users($uName,GRIMACES) grimaces" $nick $channel
}

proc atlantis::kickStats {nick uhost handle channel knick reason} {
  variable users; global atlantis
  
  # Ajoute un kick aux stats d'un utilisateur et un 'est kick' aux stats de l'autre
  
  set uName [getUserName $nick $uhost $channel]
  set uNameKicked [getUserName $knick [getchanhost $knick $channel] $channel]
  
  if { ![isStatsChan $channel] } { return }
  
  if { [userExists $uName] } {
    incr users($uName,KICK)
    set users($uName) [unixtime]
  }
  
  if { [userExists $uNameKicked] } {
    incr users($uNameKicked,KICKED)
    set users($uName) [unixtime]
  }
}

proc atlantis::topicStats {nick uhost handle channel text} {
  variable users; global atlantis
  
  # Gre les statistiques pour le changemnt de Topic
  
  set uName [getUserName $nick $uhost $channel]
  
  if { ![isStatsChan $channel] } { return }
  
  if { [userExists $uName] } {
    incr users($uName,TOPIC)
    set users($uName) [unixtime]
  }
}

proc atlantis::pubStatsInfos {nick uhost handle channel arg} {
  variable users; global atlantis
  
  # Affiche des informations sur les statistiques
  
  if {!$atlantis(allow_pubcommands)} { return }
  if ![isStatsChan $channel] { return }
  if [info exists antiflood(cmd,$nick)] { return }
  
  set numUsers [llength [array names users *[string tolower $channel]]]
  
  sendText "\002$numUsers\002 utilisateurs reconnus sur ce canal, Atlantis Stats v$atlantis(version)\
  - Plus d'infos sur: http://www.eggdrop-france.net" $nick $channel
  
  set antiflood(cmd,$nick) 1
  utimer $atlantis(antiflood) [list "unset" "::antiflood(cmd,$nick)"]
}

proc atlantis::nickStats {nick uhost handle channel newnick} {
  global antiflood atlantis
  
  # Gre les changements de nicks pour l'antiflood de commande
  
  if { [info exists antiflood(cmd,$nick)] && ![info exists antiflood(cmd,$newnick)] } {
    set antiflood(cmd,$newnick) 1
    utimer $atlantis(antiflood) [list "unset" "::antiflood(cmd,$newnick)"]
  }
}

proc atlantis::pubTop {nick uhost handle channel arg} {
  global atlantis antiflood stats_corresp
  
  # Affiche le Top10 des utilisateurs sur le canal, commande rgie par
  # l'antiflood.
  
  if [info exists antiflood(cmd,$nick)] { return }
  
  set antiflood(cmd,$nick) 1
  utimer $atlantis(antiflood) [list "unset" "::antiflood(cmd,$nick)"]
  
  if {$arg == ""} { set arg "lignes" }
  
  set arg [string toupper $arg]
  
  if { ![info exists stats_corresp($arg)] } {
    set index $stats_corresp(LIGNES)
    set arg lignes
  } else { set index $stats_corresp($arg) }
  
  set Top10Users [getTop [string tolower $channel] 9 $arg]
  set i 1
  append toReturn "Top10([string tolower $arg]): "
  
  foreach user $Top10Users {
    append toReturn "$i. [lindex $user 0] ([lindex $user $index]) "
    incr i
  }
  
  sendText $toReturn $nick $channel
}

proc atlantis::sendText {texte nick channel} {
  global atlantis
  
  # Envoi du texte  l'utilisateur en fonction de la mthode choisie dans
  # la configuration.
  
  switch -- $atlantis(say) {
    "CHAN" { set command "PRIVMSG $channel :" }
    "PRIVMSG" { set command "PRIVMSG $nick :" }
    "NOTICE" {set command "NOTICE $nick :" }
  }
  
  putquick [append command $texte]
}

proc atlantis::isStatsChan {channel} {
  global atlantis
  
  # Retourne 1 si les statistiques sont actives sur ce canal, 0 si elles ne
  # le sont pas.
  
  if { [lsearch -exact $atlantis(channels) [string tolower $channel]] == -1} {
    return 0
  }
  
  return 1
}

proc atlantis::timerStats { a b c d e } {

  # Gnre le fichier HTML et sauvegarde les statistiques
  
  saveStats
  makeHTML
}

proc atlantis::makeHTML {{statsdir ""}} {
  variable users; global atlantis; global stats_corresp
  
  # Gnre une page HTML  partir des statistiques enregistres
  # Cette page doit tre gnre assez rgulirement (tte les 10 mins par exemple)
  # Le design est dfini par un fichier CSS, vous pouvez donc le modifier comme
  # vous le souhaitez.
  
  set template_fid [open $atlantis(template_file)]
  set template_data [read $template_fid]
  close $template_fid
  
  foreach channel $atlantis(channels) {
    set statsTable [list]
    set nicksTable [list]
    
    if {$statsdir == ""} {
      set statsdir $atlantis(html_dir)
    }
    
    if ![validchan $channel] {return 0}
    
    set statsFile [file join $statsdir "[string range $channel 1 end].html"]
    set outText [list]
    
    set chanUsers [array names users "*$channel"]
    set chanTopic [stripcodes bcruag [topic $channel]]
    set chanModes [getchanmode $channel]
    set chanNicks [chanlist $channel]
    set chanNumUsers [llength $chanNicks]
    
    # - Top30 des utilisateurs et stats diverses
    lappend statsTable {<table border="0" align="center" cellspacing="0">} {<tr>} {<td>&nbsp;</td>}
    lappend statsTable {<td class="stats_title">Nick</td>}
    lappend statsTable {<td class="stats_title">Lignes</td>}
    lappend statsTable {<td class="stats_title">Mots</td>}
    lappend statsTable {<td class="stats_title">Lettres</td>}
    lappend statsTable {<td class="stats_title">Sourires</td>}
    lappend statsTable {<td class="stats_title">Grimaces</td>}
    
    # News: v 0.4
    lappend statsTable {<td class="stats_title">Kick</td>}
    lappend statsTable {<td class="stats_title">Kick</td>}
    lappend statsTable {<td class="stats_title">Topic</td>} {</tr>}
    
    
    set i 1
    foreach user [getTop $channel 29] {
      set style_num "stats_num"
      set style "stats_item"
      
      if [regexp -- {[02468]$} $i] {
        set style_num "stats_num_light"
        set style "stats_item_light"
      }
      
      lappend statsTable {<tr>}
      lappend statsTable "<td class=\"$style_num\">$i</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(PSEUDO)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(LIGNES)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(MOTS)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(LETTRES)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(SOURIRES)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(GRIMACES)]</td>"
      
      # News: v 0.4
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(KICK)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(KICKED)]</td>"
      lappend statsTable "<td class=\"$style\">[lindex $user $stats_corresp(TOPIC)]</td>"
      
      lappend statsTable {</tr>}
      
      incr i
    }
    
    set happyUser [split [join [getTop $channel 0 sourires]]]
    set sadUser [split [join [getTop $channel 0 grimaces]]]
    set qUser [split [join [getTop $channel 0 questions]]]
    set mechantUser [split [join [getTop $channel 0 kick]]]
    set malaimeUser [split [join [getTop $channel 0 kicked]]]
    set spammeurUser [split [join [getTop $channel 0 url]]]
    
    
    lappend statsTable {<tr>} {<td colspan="10">&nbsp;</td>} {</tr>}
    lappend statsTable {<tr>} {<td colspan="10" class="stats_divers_title">Autres statistiques</td>} {</tr>}
    
    lappend statsTable {<tr>}
    lappend statsTable "<td colspan=\"10\" class=\"stats_divers\"><strong>[lindex $happyUser 0]</strong> est tr&egrave;s heureux, il a un total de <strong>[lindex $happyUser $stats_corresp(SOURIRES)]</strong> sourires ! </td>"
    lappend statsTable {</tr>}
    
    lappend statsTable {<tr>}
    lappend statsTable "<td colspan=\"10\" class=\"stats_divers\">Pauvre <strong>[join [lindex $sadUser $stats_corresp(PSEUDO)]]</strong>, dprime en vue avec <strong>[lindex $sadUser $stats_corresp(GRIMACES)]</strong> grimaces...</td>"
    lappend statsTable {</tr>}
    
    lappend statsTable {<tr>}
    lappend statsTable "<td colspan=\"10\" class=\"stats_divers\">Dans le genre curieux on a <strong>[join [lindex $sadUser $stats_corresp(PSEUDO)]]</strong> avec <strong>[lindex $qUser $stats_corresp(QUESTIONS)]</strong> questions !</td>"
    lappend statsTable {</tr>}
    
    lappend statsTable {<tr>}
    lappend statsTable "<td colspan=\"10\" class=\"stats_divers\">Aussi mchant que <strong>[join [lindex $mechantUser $stats_corresp(PSEUDO)]]</strong> c'est impossible ! <strong>[lindex $mechantUser $stats_corresp(KICK)]</strong> kicks  son actif !</td>"
    lappend statsTable {</tr>}
    
    lappend statsTable {<tr>}
    lappend statsTable "<td colspan=\"10\" class=\"stats_divers\">Notre nouveau calimro est la: <strong>[join [lindex $malaimeUser $stats_corresp(PSEUDO)]]</strong> a t kick <strong>[lindex $malaimeUser $stats_corresp(KICKED)]</strong> fois !</td>"
    lappend statsTable {</tr>}
    
    lappend statsTable {<tr>}
    lappend statsTable "<td colspan=\"10\" class=\"stats_divers\"><strong>[join [lindex $spammeurUser $stats_corresp(PSEUDO)]]</strong> a t lu spammeur de l'anne avec <strong>[lindex $spammeurUser $stats_corresp(URL)]</strong> URL cites !</td>"
    lappend statsTable {</tr>}
    
    lappend statsTable {<tr>} "<td colspan=\"10\" class=\"stats_divers_title\">Il y a un total de <strong>[llength [array names users [string tolower *$channel]]]</strong> utilisateurs enregistrs! </td>" {</tr>}
    
    lappend statsTable "</table>"
    
    set statsTable [join $statsTable "\n"]
    
    # - Gnration du tableau des Nicks
    
    set opUsers ""; set halfopUsers ""; set voiceUsers ""; set normalUsers ""
    
    foreach nick $chanNicks {
      
      if [isop $nick $channel] { lappend opUsers $nick } \
      elseif [ishalfop $nick $channel] { lappend halfopUsers $nick } \
      elseif [isvoice $nick $channel] { lappend voiceUsers $nick } \
      else { lappend normalUsers $nick }
    }
    
    lappend nicksTable {<table border="0" align="center" cellspacing="0">}
    lappend nicksTable {<tr>} {<td class="stats_num"><div align="center">@</div></td>}
    lappend nicksTable "<td class=\"stats_item\">[join $opUsers {, }]</td>" {</tr>}
  
    lappend nicksTable {<tr>} {<td class="stats_num_light"><div align="center">%</div></td>}
    lappend nicksTable "<td class=\"stats_item_light\">[join $halfopUsers {, }]</td>" {</tr>}
    
    lappend nicksTable {<tr>} {<td class="stats_num"><div align="center">+</div></td>}
    lappend nicksTable "<td class=\"stats_item\">[join $voiceUsers {, }]</td>" {</tr>}
    
    lappend nicksTable {<tr>} {<td class="stats_num_light"><div align="center"></div></td>}
    lappend nicksTable "<td class=\"stats_item_light\">[join $normalUsers {, }]</td>" {</tr>} {</table>}
    
    set nicksTable [join $nicksTable "\n"]
    
    # - Remplacement des tags
    
    set date [clock format [unixtime] -format "%d/%m/%y"]
    set time [clock format [unixtime] -format "%H:%M:%S"]
    
    set outText [string map [list "&STATSTABLE&" "$statsTable" "&USERSTABLE&" $nicksTable] $template_data]
    set outText [string map [list "&CHANNAME&" $channel "&CHANTOPIC&" $chanTopic] $outText]
    set outText [string map [list "&CHANMODES&" $chanModes "&NUMUSERS&" $chanNumUsers] $outText]
    set outText [string map [list "&STATSVER&" $atlantis(version) "&STATSAUTHOR&" Merwin] $outText]
    set outText [string map [list "&DATE&" $date "&TIME&" $time] $outText]
    
    set fileID [open $statsFile w+]
    puts $fileID $outText
    close $fileID
  }
}

proc atlantis::getTop {channel {top 9} {option "lignes"}} {
  variable users; global atlantis stats_corresp
  
  # Retourne une liste contenant les informations du top10 d'un canal, chaque
  # item de la liste est de la forme dfinie dansatlantis(users_stats)
  # La liste est trie en fonction de l'argument qui peut tre un des suivants:
  # lignes, mots, lettres, sourires, grimaces, kick, kicked, questions, topic
  
  set chanUsers [array names users [string tolower *$channel]]
  set toReturn [list]
  set option [string toupper $option]
  
  if { ![info exists stats_corresp($option)] } {
    set indexTopType $stats_corresp(LIGNES)
  } else {
    set indexTopType $stats_corresp($option)
  }
  
  foreach u $chanUsers {
    set stats [list]
    
    foreach stats_type $atlantis(users_stats) {
      lappend stats $users($u,$stats_type)
    }
    
    lappend toReturn $stats
  }
  
  set toReturn [lsort -integer -decreasing -index $indexTopType $toReturn]
  set toReturn [lrange $toReturn 0 $top]
  
  return $toReturn
}

proc atlantis::freeStats {} {
  variable users, global atlantis
  
  # Supprimme toutes les entres inutiles en mmoire, celles dont le canal
  # n'est plus monitor mais aussi celle dont on a pas vu l'utilisateur
  # depuis X jours (dfinis plus haut).
  
  
}

atlantis::scriptInit
putlog "Atlantis Stats v$atlantis(version) - Actif sur \002[llength $atlantis(channels)]\002 can[expr {([llength $atlantis(channels)] <= 1) ? [format al] : [format aux]}] ([join $atlantis(channels) ", "])"




